local Libra = LibStub("Libra")

local addon = Libra:NewAddon(...)

local professions = {}

local function sortNames(a, b)
	if a.skill == b.skill then
		return a.name < b.name
	else
		return a.skill > b.skill
	end
end

local processing

local function update(self)
	if processing then return end
	
	processing = true
	
	local collapsed = {}
	-- we can only retrieve data on visible entries, so need to show offline members
	local showOffline = GetGuildRosterShowOffline()
	SetGuildRosterShowOffline(true)

	local i = 1

	while true do
		local skillID, isCollapsed, _, headerName, _, _, numPlayers, playerDisplayName, playerFullName, _, online, _, skill = GetGuildTradeSkillInfo(i)
		if not skillID then
			break
		end
		if playerFullName then
			table.insert(professions[skillID], {name = playerFullName, skill = skill, online = online})
		elseif headerName and numPlayers > 0 then
			-- also need to expand each profession
			ExpandGuildTradeSkillHeader(skillID)
			collapsed[skillID] = isCollapsed
			professions[skillID] = professions[skillID] or {}
			wipe(professions[skillID])
		end
		i = i + 1
	end

	SetGuildRosterShowOffline(showOffline)
	if GuildRoster_Update then GuildRoster_Update() end -- guild roster seems to bug slightly (while open) if we don't do this
	
	for k, v in pairs(professions) do
		if collapsed[k] then
			CollapseGuildTradeSkillHeader(k)
		end
		sort(v, sortNames)
	end
	
	processing = false
	
	self:UnregisterEvent("GUILD_ROSTER_UPDATE")
	self:UnregisterEvent("GUILD_TRADESKILL_UPDATE")
end

addon.GUILD_ROSTER_UPDATE = update
addon.GUILD_TRADESKILL_UPDATE = update

local function onHeaderClick(self)
	ViewGuildRecipes(self.value)
end

local function onClick(self, skillID)
	GetGuildMemberRecipes(self.value, skillID)
end

local dropdown = Libra:CreateDropdown("Menu")
dropdown.xOffset = 0
dropdown.yOffset = 0
dropdown.initialize = function(self, level)
	if level == 1 then
		local showOffline = GetGuildRosterShowOffline()
		SetGuildRosterShowOffline(true)

		local i = 1

		while true do
			local skillID, _, _, headerName, _, numPlayers = GetGuildTradeSkillInfo(i)
			if not skillID then
				break
			end
			if headerName and numPlayers > 0 then
				local info = UIDropDownMenu_CreateInfo()
				info.text = headerName
				info.value = skillID
				info.func = onHeaderClick
				info.hasArrow = true
				info.notCheckable = true
				self:AddButton(info, level)
			end
			i = i + 1
		end

		SetGuildRosterShowOffline(showOffline)
		if GuildRoster_Update then GuildRoster_Update() end
		
		addon:RegisterEvent("GUILD_ROSTER_UPDATE")
		addon:RegisterEvent("GUILD_TRADESKILL_UPDATE")
		
		GuildRoster()
		QueryGuildRecipes()
	elseif level == 2 then
		local skillID = UIDROPDOWNMENU_MENU_VALUE
		local canView = CanViewGuildRecipes(skillID)
		professions[skillID] = professions[skillID] or {}
		for i, v in ipairs(professions[skillID]) do
			local info = UIDropDownMenu_CreateInfo()
			info.text = format("%s (%d)", Ambiguate(v.name, "guild"), v.skill)
			info.value = v.name
			info.func = onClick
			info.arg1 = skillID
			if canView then
				info.colorCode = v.online and GREEN_FONT_COLOR_CODE
			else
				info.notClickable = true
				info.disablecolor = v.online and GREEN_FONT_COLOR_CODE
			end
			info.notCheckable = true
			self:AddButton(info, level)
		end
	end
end

LibStub("LibDataBroker-1.1"):NewDataObject("GuildProfessions", {
	type = "launcher",
	label = "Guild Professions",
	icon = [[Interface\Icons\Achievement_General_WorkingasaTeam]],
	OnClick = function(self)
		dropdown:Toggle(nil, self)
	end,
})